

-- BetterCombat Attack Animation Compatibility script
-- v1.4 (7th March, 2024)
-- changelog from previous version (v1.3):
-- 	- Fixed typo in hard-disable check causing script to error out.

-- This script disables custom item pivots when attacking with a BetterCombat weapon, and while holding a two-hander with a custom idle.
-- To use: Check the config variables below - you'll need to set your pivot paths, and maybe some weaponry item tags to include whatever mods you're playing with.
-- It's not perfect - BetterCombat doesn't 'swing' the player arm during its custom attacks, so it needs to be detected in other, stupider ways.

-- The script will disable itself if BetterCombat isn't installed on the client.
-- Here are some methods for manually controlling the script:
-- Functions: 
-- - function BCC_SetEnabled(bool)
-- 		Enable or disable this script.
-- - function BCC_DisableForTicks(int)
-- 		Disable this script for the given number of ticks.
-- Variables:
-- - setting BCC_bDisableScript to true will disable the script.

-- Modify however you want, just give me credit if you release it.
-- written by pont
-- visit pont.cool ;3


-- #####################################################################################################
-- Variable Declarations
-- #####################################################################################################

-- Config
BCC_bDisableScript = true									-- Config: An easy way to completely disable this script's behaviour. Overrides enable/disable functions.
local RightPivot = models.pinkwolf.RightArm.RightItemPivot	-- Config: Avatar's right item pivot.
local LeftPivot = models.pinkwolf.LeftArm.LeftItemPivot		-- Config: Avatar's left item pivot.

BCC_bRightPivotEnabled = true								-- Readout: State of right item pivot
BCC_bLeftPivotEnabled = true								-- Readout: State of left item pivot

local WeaponTags = {		-- Config: Item tags animated by BetterCombat
	-- Vanilla weapon tags
	["minecraft:swords"] = {},
	["minecraft:spears"] = {},
	["minecraft:axes"] = {},
	-- Common weapon tags (fabric)
	["c:swords"] = {},
	["c:spears"] = {},
	["c:axes"] = {},
	-- BetterCombat weapon tags (should hit everything custom config'd into BC)
	["bettercombat:anchor"] = {}                      ,
	["bettercombat:axe"] = {}                      	  ,
	["bettercombat:battlestaff"] = {}                 ,
	["bettercombat:bow_two_handed_heavy"] = {}        ,
	["bettercombat:bow_two_handed_light"] = {}        ,
	["bettercombat:claw"] = {}                        ,
	["bettercombat:claymore"] = {}                    ,
	["bettercombat:coral_blade"] = {}                 ,
	["bettercombat:crossbow_two_handed_heavy"] = {}   ,
	["bettercombat:crossbow_two_handed_light"] = {}   ,
	["bettercombat:cutlass"] = {}                     ,
	["bettercombat:dagger"] = {}                      ,
	["bettercombat:double_axe"] = {}                  ,
	["bettercombat:fist"] = {}                        ,
	["bettercombat:glaive"] = {}                      ,
	["bettercombat:halberd"] = {}                     ,
	["bettercombat:hammer"] = {}                      , -- sorry. Sorry
	["bettercombat:heavy_axe"] = {}                   ,	-- i forgot to put commas on these as i was writing them
	["bettercombat:katana"] = {}                      ,	-- its fucking ugly i KNOW IM SORRY
	["bettercombat:lance"] = {}                       ,
	["bettercombat:mace"] = {}                        ,
	["bettercombat:pickaxe"] = {}                     ,
	["bettercombat:rapier"] = {}                      ,
	["bettercombat:scythe"] = {}                      ,
	["bettercombat:sickle"] = {}                      ,
	["bettercombat:soul_knife"] = {}                  ,
	["bettercombat:spear"] = {}                       ,
	["bettercombat:staff"] = {}                       ,
	["bettercombat:sword"] = {}                       ,
	["bettercombat:trident"] = {}                     ,
	["bettercombat:twin_blade"] = {}				  ,
	["bettercombat:wand"] = {}						  ,
	-- SimplySwords weaponry
	["simplyswords:swords"] = {},
	-- Tinker's Construct (there's a datapack for BetterCombat config floating around somewhere)
	["tconstruct:modifiable/melee"] = {}
	
}

local WeaponIDs = {			-- Config: Item IDs of specific weapons, for edge-cases that aren't tagged sensibly.
	-- Vanilla weaponry (For versions below 1.19.4, in which they weren't yet tagged! What fun!)
	["minecraft:wooden_sword"] = {},
	["minecraft:stone_sword"] = {},
	["minecraft:stone_sword"] = {},
	["minecraft:iron_sword"] = {},
	["minecraft:golden_sword"] = {},
	["minecraft:diamond_sword"] = {},
	["minecraft:netherite_sword"] = {},
	
	["minecraft:wooden_axe"] = {},
	["minecraft:stone_axe"] = {},
	["minecraft:stone_axe"] = {},
	["minecraft:iron_axe"] = {},
	["minecraft:golden_axe"] = {},
	["minecraft:diamond_axe"] = {},
	["minecraft:netherite_axe"] = {},
	
	["minecraft:trident"] = {}
}

local TwoHandedWeaponTags = {		-- Config: Two-handed BetterCombat animation sets
	-- BetterCombat two-handed weapon tags (should hit everything custom config'd into BC)
	["bettercombat:anchor"] = {}                      ,
	["bettercombat:battlestaff"] = {}                 ,
	["bettercombat:bow_two_handed_heavy"] = {}        ,
	["bettercombat:bow_two_handed_light"] = {}        ,
	["bettercombat:claymore"] = {}                    ,
	["bettercombat:crossbow_two_handed_heavy"] = {}   ,
	["bettercombat:crossbow_two_handed_light"] = {}   ,
	["bettercombat:double_axe"] = {}                  ,
	["bettercombat:glaive"] = {}                      ,
	["bettercombat:halberd"] = {}                     ,
	["bettercombat:hammer"] = {}                      ,
	["bettercombat:heavy_axe"] = {}                   ,
	["bettercombat:katana"] = {}                      ,
	["bettercombat:scythe"] = {}                      ,
	["bettercombat:soul_knife"] = {}                  ,
	["bettercombat:spear"] = {}                       ,
	["bettercombat:staff"] = {}                       ,
	["bettercombat:twin_blade"] = {}				  
}

local TwoHandedWeaponIDs = {		-- Config: Item IDs of specific two-handed weapons, for edge-cases that aren't tagged sensibly.
	-- Bespoke rules for literally all of SimplySwords' two-handers. No, seriously. Every weapon in that mod is tagged 'simplyswords:swords'.
	["simplyswords:iron_katana"] = {},
	["simplyswords:iron_glaive"] = {},
	["simplyswords:iron_claymore"] = {},
	["simplyswords:iron_greathammer"] = {},
	["simplyswords:iron_greataxe"] = {},
	["simplyswords:iron_scythe"] = {},
	["simplyswords:iron_halberd"] = {},
	["simplyswords:iron_warglaive"] = {},	-- one-hander with idle animation
	
	["simplyswords:gold_katana"] = {},
	["simplyswords:gold_glaive"] = {},
	["simplyswords:gold_claymore"] = {},
	["simplyswords:gold_greathammer"] = {},
	["simplyswords:gold_greataxe"] = {},
	["simplyswords:gold_scythe"] = {},
	["simplyswords:gold_halberd"] = {},
	["simplyswords:gold_warglaive"] = {},	-- one-hander with idle animation
	
	["simplyswords:diamond_katana"] = {},
	["simplyswords:diamond_glaive"] = {},
	["simplyswords:diamond_claymore"] = {},
	["simplyswords:diamond_greathammer"] = {},
	["simplyswords:diamond_greataxe"] = {},
	["simplyswords:diamond_scythe"] = {},
	["simplyswords:diamond_halberd"] = {},
	["simplyswords:diamond_warglaive"] = {},	-- one-hander with idle animation
	
	["simplyswords:netherite_katana"] = {},
	["simplyswords:netherite_glaive"] = {},
	["simplyswords:netherite_claymore"] = {},
	["simplyswords:netherite_greathammer"] = {},
	["simplyswords:netherite_greataxe"] = {},
	["simplyswords:netherite_scythe"] = {},
	["simplyswords:netherite_halberd"] = {},
	["simplyswords:netherite_warglaive"] = {},	-- one-hander with idle animation
	
	["simplyswords:runic_katana"] = {},
	["simplyswords:runic_glaive"] = {},
	["simplyswords:runic_claymore"] = {},
	["simplyswords:runic_greathammer"] = {},
	["simplyswords:runic_greataxe"] = {},
	["simplyswords:runic_scythe"] = {},
	["simplyswords:runic_halberd"] = {},
	["simplyswords:runic_warglaive"] = {},	-- one-hander with idle animation
	
	["simplyswords:brimstone_claymore"] = {},
	["simplyswords:watcher_claymore"] = {},
	["simplyswords:hearthflame"] = {},
	["simplyswords:soulkeeper"] = {},
	["simplyswords:twisted_blade"] = {},
	["simplyswords:soulrender"] = {}, -- one-hander with idle animation
	["simplyswords:soulpyre"] = {},
	["simplyswords:icewhisper"] = {},
	["simplyswords:arcanethyst"] = {},
	["simplyswords:thunderbrand"] = {},
	["simplyswords:slumbering_lichblade"] = {},
	["simplyswords:waking_lichblade"] = {},
	["simplyswords:awakened_lichblade"] = {},
	["simplyswords:whisperwind"] = {},				-- Please appreciate how annoying this was to configure.
	
	-- Tinker's Construct (there's a datapack for BetterCombat config floating around somewhere)
	["tconstruct:cleaver"] = {},
	["tconstruct:scythe"] = {},
	["tconstruct:broad_axe"] = {},
	["tconstruct:vein_hammer"] = {},
	["tconstruct:sledge_hammer"] = {}
}

local PostAttackDuration = 9				-- Config: Extra ticks to keep custom pivots disabled after a weapon's attack cooldown is over

-- Internal
local HeldItemTags							-- Tags for the currently held item
local AttackingTicker = 0					-- Ticks left in current attack. On 0, attack is over - will continue ticking negative by amount defined by PostAttackDuration.
local AttackKey								-- Client's attack keybind

local LastTickHeldItem						-- ItemStack in mainhand on last tick
local ItemSwitchCooldown = 0				-- Time before attacking is ready on newly-switched-to item
local LastItemDelay = 0						-- Last tick's held item's charged attack delay. This lags behind a few ticks after a player switches weapons. Never easy, is it?

local bRightPivotCurTick = true				-- Current tick's 'is enabled' bool for Right pivot
local bLeftPivotCurTick = true				-- Current tick's 'is enabled' bool for Left pivot

local bRightPivotPrevTick = bRightPivotCurTick		-- Last tick's 'is enabled' bool for Right pivot
local bLeftPivotPrevTick = bLeftPivotCurTick		-- Last tick's 'is enabled' bool for Left pivot

local ScriptDisableTicker = 0				-- Disables the script for given amount of ticks. 0 is enabled. -1 is disabled forever until told otherwise.


-- #####################################################################################################
-- Script Proper
-- #####################################################################################################

-- Control whether the script is enabled or disabled.
-- Useful if you need to turn this script off during some cool custom animations.
-- This overrides any disabled duration previously set.
function BCC_SetEnabled(in_IsEnabled)
	if in_isEnabled then
		-- Enabled!
		ScriptDisableTicker = 0
		bRightPivotCurTick = false
		bLeftPivotCurTick = false
	else
		-- Disabled!
		ScriptDisableTicker = -1
		-- Unhide pivots
		bRightPivotCurTick = true
		bLeftPivotCurTick = true
	end	
end
-- Disable the script for a given number of ticks.
function BCC_DisableForTicks(in_Ticks)
	ScriptDisableTicker = math.floor(in_Ticks)
	-- Unhide pivots
	bRightPivotCurTick = true
	bLeftPivotCurTick = true
end


-- Pre-ping dispatch (CALL THIS INSTEAD OF BCC_PivotPing !!!)
-- Checks for state deltas before sending out a ping.
function BCC_CheckForPivotDeltas()

	--log("R", (bRightPivotPrevTick ~= bRightPivotCurTick), "L", (bLeftPivotPrevTick ~= bLeftPivotCurTick)) --debug
	--log("R", bRightPivotCurTick, "L", bLeftPivotCurTick)

	-- check for delta
	if bRightPivotPrevTick ~= bRightPivotCurTick or bLeftPivotPrevTick ~= bLeftPivotCurTick then
		-- there is a delta: network new info
		pings.BCC_PivotPing(bRightPivotCurTick, bLeftPivotCurTick)
	end

end

--pingcount = 0 --debug
-- Ping dispatch (Keystrokes, sensibly, aren't networked)
function pings.BCC_PivotPing(in_EnableRight, in_EnableLeft)

	--log("PING!", pingcount,"R", in_EnableRight, "L", in_EnableLeft) --debug
	--pingcount = pingcount + 1 --debug
	
	-- Affect pivots
	-- Right
	RightPivot:setVisible(in_EnableRight) -- Disable pivot itself
	BCC_bRightPivotEnabled = in_EnableRight -- Affect readout var (main logic doesn't run on remote avatars)
	-- Left
	LeftPivot:setVisible(in_EnableLeft) -- Disable pivot itself
	BCC_bLeftPivotEnabled = in_EnableLeft -- Affect readout var (main logic doesn't run on remote avatars)
end

-- On load (init function)
-- Bound to .init within Figura's Events API
function events.entity_init()

	-- Force script to disable when BetterCombat is not detected
	if not client:isModLoaded("bettercombat") then
		BCC_bDisableScript = true
		return
	end
	
	-- Get attack binding
	AttackKey = keybinds:fromVanilla("key.attack")
end

-- Every tick (update function)
-- Bound to .tick within Figura's Events API
function events.tick()
	-- Check config hard disable
	if BCC_bDisableScript == true then return end
	
	-- Check if running on remote (non-client) avatar
	if not host:isHost() then return end -- Return immediately - the script relies on reading inputs, which aren't networked. Rely instead on BCC_PivotPing dispatches from foreign clients.

	-- Script disable ticker
	-- Greater than 0 (waiting to reenable)?
	if ScriptDisableTicker > 0 then
		-- Decrement ticker and skip checks.
		ScriptDisableTicker = ScriptDisableTicker - 1
		goto SkipToFinalChecks
	-- Fewer than 0 (disabled forever?)
	elseif ScriptDisableTicker < 0 then
		-- Return, doing nothing.
		return
	end
	

	-- Check for item switch, so clicking before the weapon's ready won't start the AttackingTicker logic.
	-- Doesn't proc on switching between identical items - but that's fine, as that doesn't cause item-switch weapon warmup anyway.
	if LastTickHeldItem ~= player:getHeldItem().id then
		ItemSwitchCooldown = math.floor(player:getChargedAttackDelay())
	end
	-- Check for item warmup actually updating, because it lags behind a few ticks (for fuck's sake).
	-- This won't proc on weapons with the same cooldown, but that's okay, because it'll work anyway in that case! Woo!
	if LastItemDelay ~= player:getChargedAttackDelay() then
		-- Adjust the cooldown ticker, keeping the same tickdown progress.
		ItemSwitchCooldown = math.max(math.floor(player:getChargedAttackDelay() - (LastItemDelay - ItemSwitchCooldown)), 0)
		-- Update last item delay ready for the next item switch
		LastItemDelay = player:getChargedAttackDelay()
		--log("Switched!", math.random(1,100)) -- debug
	end
	--log(player:getChargedAttackDelay(), ItemSwitchCooldown) -- debug
	
	-- Get mainhand held item's tags
	HeldItemTags = player:getHeldItem():getTags()
				
	-- Check if weapon is two-handed weaponry
	for i = 1, #HeldItemTags, 1 do
		if TwoHandedWeaponTags[HeldItemTags[i]] or TwoHandedWeaponIDs[player:getHeldItem():getID()] then
			-- Weaponry is two-handed - disable both pivots, dispatch, and skip the one-hand checks.
			--log("two handed!") -- debug
			bRightPivotCurTick = false
			bLeftPivotCurTick = false
			goto SkipToFinalChecks
		end
	end
	
	-- At this point, we know the player's not holding a two-hander weapon.
	-- Proceed ahead with rest of logic
	
	-- BetterCombat doesn't swing the player's arm during attacks - have to check other things. Blegh!
	-- If pressing attack input, and not already attacking, and not in item-switch cooldown...
	if AttackKey:isPressed() and AttackingTicker <= 0 and ItemSwitchCooldown <= 0 then
		-- Check if item is BC weaponry
		
		-- Checks for MAIN HAND
		for i = 1, #HeldItemTags, 1 do
			--log(HeldItemTags[i])
			if WeaponTags[HeldItemTags[i]] or WeaponIDs[player:getHeldItem():getID()] then
				-- Set attacking ticker to weapon's attack delay
				AttackingTicker = math.floor(player:getChargedAttackDelay())
				-- Disable the right hand pivot, letting BC handle the item transform while attacking.
				bRightPivotCurTick = false
				
				--log("Attack!") -- debug
				--log(player:getHeldItem().id,  player:getHeldItem(true).id) -- debug
				goto R_Done
			end
		end
		::R_Done::
		-- Checks for OFFHAND
		-- Get offhand held item's tags
		HeldItemTags = player:getHeldItem(true):getTags()
		-- Check if item is BC weaponry
		for i = 1, #HeldItemTags, 1 do
			if WeaponTags[HeldItemTags[i]] or WeaponIDs[player:getHeldItem(true):getID()] then
				--log("weapon (offhand)!")
				-- Don't set ticker - BetterCombat silently swaps the current attacking weapon's data into the main hand while dual-wielding, so the first loop procs everything.
				-- This loop only exists for hiding the left item pivot.
				
				-- Disable the left hand pivot, letting BC handle the item transform while attacking.
				bLeftPivotCurTick = false
				
				goto L_Done
			end
		end
		::L_Done::
	end
	--log(AttackingTicker) -- debug
	
	-- Reenable item pivots when done attacking.
	if AttackingTicker <= 0 - PostAttackDuration then
		bRightPivotCurTick = true
		bLeftPivotCurTick = true
	end
	
	-- Decrement AttackingTicker
	if AttackingTicker > 0 - PostAttackDuration then AttackingTicker = AttackingTicker - 1 end
	-- Decrement ItemSwitchCooldown
	if ItemSwitchCooldown > 0 then ItemSwitchCooldown = ItemSwitchCooldown - 1 end
	
	-- goto destination for dual-hander logic
	::SkipToFinalChecks::
	
	-- Check for deltas and ping (spam-safe)
	BCC_CheckForPivotDeltas()
	
	-- Update last tick held item tracker.
	LastTickHeldItem = player:getHeldItem().id
	-- Update enable state delta trackers
	bRightPivotPrevTick = bRightPivotCurTick
	bLeftPivotPrevTick = bLeftPivotCurTick
	
	-- Update readout vars
	BCC_bRightPivotEnabled = bRightPivotCurTick
	BCC_bLeftPivotEnabled = bLeftPivotCurTick
end