


-- hide stock playermodel parts
-- body (inner layer, outer layer, cape)
vanilla_model.PLAYER:setVisible(false)
-- armor
vanilla_model.ARMOR:setVisible(false)
-- we're NOT hiding held items, elytra, parrots
-- specifically unhide helmet item (pumpkins, playerheads, etc)
vanilla_model.HELMET_ITEM:setVisible(true)


-- VARIABLES
-- Some of these are global so they can be read from places like driving functions within the animationsystem.
-- Some things are hardcoded across the script, because this is personal work and we can't be arsed.

-- Misc
local ActiveItem;				-- Local cached version of player:getActiveItem()
local DefaultNameText = nameplate.CHAT:getText()	-- Default player nameplate string

-- Audio
local AudioVolume = 1			-- Config: Range of dispatched audio. Engine calls this Volume, but it actually affects the falloff instead.
local AudioWolfPitchMin = 0.75	-- Config: Lowest possible pitch of avatar wolf sounds
local AudioWolfPitchMax = 0.95	-- Config: Highest possible pitch of avatar wolf sounds

local BarkTicker = -1			-- Decrementing timer used to control barking. Decrements each tick, plays sound on 0. When nonzero, bumped up by other tickers expiring as to not overlap.
local GrowlTicker = -1			-- Decrementing timer used to control growling. Decrements each tick, plays sound on 0. When nonzero, bumped up by other tickers expiring as to not overlap.
local WhineTicker = -1			-- Decrementing timer used to control whining. Decrements each tick, plays sound on 0. When nonzero, bumped up by other tickers expiring as to not overlap.

local Sound_Howl				-- Figura sound definition for howling. Awooo!
local Sound_Shakeoff			-- Figura sound definition for wolf shakeoff

-- Tex
local Tex_Ruby01				-- Texturefile ruby_01.png
-- TEXTURE QUERY SYNTAX IS 'textures["texturename"]', without file extension. :get() doesn't work. Huh?

-- Health
local PlayerHealth = 1			-- Player's health. Init low as to not trigger on-damage stuff when reloading the avatar while hurt.
TakenDamageTicker = 0			-- Ticker that gets set when damage is taken
local PlayerMaxHealth = 20 		-- Player's max health
PlayerHealthProportion = 1 		-- Proportion of health player has, where 1 is full.
local LastTickAbsorption = 0	-- Last tick's player absorption
local AbsorptionVFXTicker = 0	-- Particle spawn ticker for absorption stuff. Everything else related is hardcoded. cba
local LowHealthWhineMin = 400	-- Min rand value for whine ticker when hurt
local LowHealthWhineMax = 1200	-- Max rand value for whine ticker when hurt

-- Armour
local eArmourMaterials = {			-- Enum of possible armour materials
	NoArmour = {},
	Leather = {},
	Chainmail = {},
	Iron = {},
	Gold = {},
	Diamond = {},
	Netherite = {},
	Turtle = {}
}
local EquippedArmourMaterials = {	-- Currently equipped armour, by material
	{},
	{},
	{},
	{}
}

-- Hunger
local PlayerHunger = 20			-- Player's hunger value.
bIsEatingOrDrinking = false		-- Player's currently eating?

-- Sprinting
local SprintTicker = 0			-- Timer for sprinting stuff.
local SprintPantTime = 200		-- Config: Time, in ticks, for avatar to be sprinting before having a chance of playing wolf panting sound

-- Jiggles (hehe! :3)
local VelocityLastTick = vec(0,0,0)	-- Last tick's player velocity
VelocityChange = vec(0,0,0)			-- Change in player velocity from last tick
local BodyYawAVelLastTick = 0		-- Last tick's body yaw angular velocity
BodyYawAVelChange = 0				-- Change in body yaw angular velocity from last tick
JiggleTickdown = vec(0,0,0)			-- Tickdown based on VelocityChange
local JiggleTickerCap = 34			-- Config: JiggleTickdown axes maximum value. Prevents large movements (teleporting, respawning, etc) causing long-duration jiggles.
local JiggleTickerCutoff = 2		-- Config: JiggleTickdown axes are set to 0 if at or below this value - this stops vibrations on slight movements.
local JiggleDurationValue = 25		-- Config: Jiggle duration modifier. Higher values jiggle for longer, and a greater distance.
JiggleIntensityValue = 1.1			-- Config: Jiggle intensity modifier. Higher values jiggle a greater distance.
JiggleAngleClamp = 10				-- Config: Jiggle angle clamping value. Rotation caps out at this.
JiggleBodyYawIntensity = 0.05		-- Config: Multiplier for xz VelocityChange drawn from Body Yaw values. Body Yaw can easily hit values around 50, so this needs a lot of dampening.

-- Shakeoff
local bWetLastTick = false			-- Was player wet last tick? Sopping? Sogged? Fucking Saturated? DRENCHED?
ShakeoffTicker = 0					-- Anim control ticker for shakeoff
local ShakeoffDuration = 35			-- Config: Shakeoff duration
local ShakeoffDelayTicker = -1		-- Delay shakeoff for a bit
local ShakeoffDelayDuration	= 41	-- Config: Time between leaving water and shaking off 
local bShakeoffFullyDrenched = false	-- Track if the player was just wading or got fully soaked
ShakeoffMoveVelThreshold = 0.05		-- Config: Horizontal velocity threshold that, above which, leg shaking will not play.
local ShakeoffDripsVFXBounds = vec(0.5,0.5,1.5,0.3) -- Config: Shakeoff drips spawn bbox - X radius, Y radius, height, height offset from player origin.
local bShakeoffDripsSlowFlipFlop = true -- Used to only pre-shakeoff-drip every other tick when not very wet
local ShakeoffArmourClinkTicker = 0 -- Controls dispatch of armour clink sfx while shaking off with armour on
local ShakeoffArmourClinkTime = 5	-- Ticks between armour clinks
local ShakeoffArmourClinkIndex = 1	-- Index of armour piece to clink (affects material sound)

-- Eyes (data)
local EyesLocation = {vec(9,11),vec(13,11)}	-- Config: Position of eyes in texture sheet.
local EyesPixelsOpen = {}			-- Pixel colours for open eyes. Read from texture on init.
local EyesPixelsClosed = {}			-- Pixel colours for closed eyes. Read from texture on init.
local EyesPixelsLeft = {}			-- Pixel colours for left-looking eyes. Read from texture on init.
local EyesPixelsRight = {}			-- Pixel colours for right-looking eyes. Read from texture on init.
local EyesPixelsHalf = {}			-- Pixel colours for half-open eyes. Read from texture on init.
local EyesPixelsHuge = {}			-- Pixel colours for big puppydog eyes. Pwease? Read from texture on init.

local EyesPixelsOpen_Uber = {}		-- As above, for Absorption effect.
local EyesPixelsLeft_Uber = {}		-- ^
local EyesPixelsRight_Uber = {}		-- ^
local EyesPixelsHalf_Uber = {}		-- ^
local EyesPixelsHuge_Uber = {}		-- ^
-- Enum for eyes poses
local eEyesPose = {
	LookForward = {},	-- Eyes are looking ahead.
	LookLeft = {},		-- Eyes are looking left.
	LookRight = {},		-- Eyes are looking right.
	HalfOpen = {},		-- Eyes are looking ahead, halfway open.
	Pwease = {}			-- Eyes are looking ahead, huge. Pleading emoji.
	-- There is no eyes shut pose. This is controlled separately, so that blinking doesn't cancel other poses.
}
local bAreEyesShut = false -- Separate eyes-are-shut tracking. Signal var - controlled by the blink shut ticker.
local bLastTickAreEyesShut = bAreEyesShut	-- Previous tick's eyes-are-shut bool
local bPwease = false						-- Control bool: Force eyes to Pwease? state
local PweaseVFXTicker = 0					-- Ticker for spawning sparkle particles during Pwease state. Everything else about this one's hardcoded, sorry chief.
-- Eyes (vars)
local EyesState = eEyesPose.LookForward;	-- Current eyes pose
local LastTickEyesState = EyesState			-- Previous tick's eyes pose
local BlinkIntervalTicker = 100				-- Decrementing timer between blinks. On 0, set BlinkShutTimer, and reset.
local BlinkShutTicker = 0					-- Decrementing timer for eyes being shut. While nonzero, eyes are closed, and BlinkIntervalTicker doesn't decrement.
local EyeChangeTicker = 0					-- Decrementing timer for eye movement cooldown. While nonzero, eye pose is not allowed to change.
local EyeChangeCooldown = 10				-- Config: Minimum amount of time between eye pose changes.
local BlinkIntervalMin = 100				-- Config: Minimum time between blinks. Realistic average is 15 blinks per minute / 4 seconds / 80 ticks.
local BlinkIntervalMax = 140				-- Config: Maximum time between blinks. Realistic average is 12 blinks per minute / 5 seconds / 100 ticks.
local BlinkDurationDay = 2					-- Config: Duration of eyes being shut during a blink, during daytime.
local BlinkDurationNight = 8				-- Config: Duration of eyes being shut during a blink, during nighttime. Sleepy...
local LookSidewaysYawVelThreshold = 10		-- Config: Player head yaw velocity threshold for setting eye pose to looking sideways
local LookSidewaysAmountThreshold = 60		-- Config: Player head needs to turn at least this amount before eyes will look sideways
local LookSidewaysAmountTicker = LookSidewaysAmountThreshold -- Ticker for tracking total recent head turn amount.

-- FUNCTIONS (HELPERS)

-- "Ooh Lua's core is so small!" your math.random() is unhelpful and you don't have math.clamp() either fuck off and fuck you
function RandomWithinRange(in_Lower, in_Higher)
	return in_Lower + (math.random()*(in_Higher-in_Lower))
end
-- RGB -> Normal (Scalar)
function NormaliseRGB (R,G,B)
	return R/255, G/255, B/255
end
-- Square a number while preserving its sign
function SquarePreserveSign(in_Value)
	if in_Value>0 then return in_Value*in_Value else return in_Value*-in_Value end
end
-- Move a number away from 0
function AwayFromZero(in_Value, in_MoveBy)
	if in_Value>0 then return in_Value+in_MoveBy else return in_Value-in_MoveBy end
end

-- FUNCTIONS (BEHAVIOUR)

-- Read two eye's worth of pixels from the given texture sheet.
-- Hardcoded to assume a 2x2 eye.
function Eyes_ReadPixels(in_LeftEyeCoord, in_RightEyeCoord, in_Tex)
	return {
			in_Tex:getPixel(in_LeftEyeCoord.x, 		in_LeftEyeCoord.y),		-- Left eye, top left
			in_Tex:getPixel(in_LeftEyeCoord.x+1, 	in_LeftEyeCoord.y),		--	top right
			in_Tex:getPixel(in_LeftEyeCoord.x, 		in_LeftEyeCoord.y+1),	--	bottom left
			in_Tex:getPixel(in_LeftEyeCoord.x+1, 	in_LeftEyeCoord.y+1),	--	bottom right
			--
			in_Tex:getPixel(in_RightEyeCoord.x, 	in_RightEyeCoord.y),	-- Right eye, top left
			in_Tex:getPixel(in_RightEyeCoord.x+1, 	in_RightEyeCoord.y), 	--	top right
			in_Tex:getPixel(in_RightEyeCoord.x, 	in_RightEyeCoord.y+1),	--	bottom left
			in_Tex:getPixel(in_RightEyeCoord.x+1, 	in_RightEyeCoord.y+1)	--	bottom right
		}
end

-- Update eyes texture. Hardcoded as all hell.
function Eyes_Update()

	-- Return immediately, unless states have changed.
	if LastTickEyesState ~= EyesState or bLastTickAreEyesShut ~= bAreEyesShut or LastTickAbsorption ~= player:getAbsorptionAmount() then return end
	
	-- Update texture
	-- Eyes shut
	if bAreEyesShut then
		Eyes_WritePixels(Tex_Ruby01, EyesPixelsClosed)			--log("shut!") 
		return
	end	
	
	-- Eyes open
	-- Are there any absorption hearts?
	if player:getAbsorptionAmount() == 0 then
		-- No; use normal eye textures
		if 		EyesState == eEyesPose.LookForward 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsOpen)		--log("forward")
		elseif	EyesState == eEyesPose.LookLeft 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsLeft)		--log("left")
		elseif	EyesState == eEyesPose.LookRight 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsRight)		--log("right")
		elseif	EyesState == eEyesPose.HalfOpen 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsHalf)		--log("half")
		elseif	EyesState == eEyesPose.Pwease 		then Eyes_WritePixels(Tex_Ruby01, EyesPixelsHuge)		--log("pwease?")
		else log("Warning! Eyes_SetState has nothing for eye pose ",in_EyesState) end
	else
		-- Yes; use alternate eye textures
		if 		EyesState == eEyesPose.LookForward 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsOpen_Uber)	--log("forward", "(uber)")
		elseif	EyesState == eEyesPose.LookLeft 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsLeft_Uber)	--log("left", "(uber)")
		elseif	EyesState == eEyesPose.LookRight 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsRight_Uber)	--log("right", "(uber)")
		elseif	EyesState == eEyesPose.HalfOpen 	then Eyes_WritePixels(Tex_Ruby01, EyesPixelsHalf_Uber)	--log("half", "(uber)")
		elseif	EyesState == eEyesPose.Pwease 		then Eyes_WritePixels(Tex_Ruby01, EyesPixelsHuge_Uber)	--log("pwease?", "(uber)")
		else log("Warning! Eyes_SetState has nothing for eye pose ",in_EyesState,"(uber)") end
	end
end

-- Write two eye's worth of given pixels, and push to GPU.
function Eyes_WritePixels(in_Tex, in_PixelData)
	--log(in_Tex, in_PixelData)
	in_Tex:setPixel(EyesLocation[1].x, 		EyesLocation[1].y,		in_PixelData[1])	-- Left eye, top left
	in_Tex:setPixel(EyesLocation[1].x+1, 	EyesLocation[1].y,		in_PixelData[2])	--	top right
	in_Tex:setPixel(EyesLocation[1].x, 		EyesLocation[1].y+1,	in_PixelData[3])	--	bottom left
	in_Tex:setPixel(EyesLocation[1].x+1, 	EyesLocation[1].y+1,	in_PixelData[4])	--	bottom right
	-- yup
	in_Tex:setPixel(EyesLocation[2].x, 		EyesLocation[2].y,		in_PixelData[5])	-- Right eye, top left
	in_Tex:setPixel(EyesLocation[2].x+1, 	EyesLocation[2].y,		in_PixelData[6]) 	--	top right
	in_Tex:setPixel(EyesLocation[2].x, 		EyesLocation[2].y+1,	in_PixelData[7])	--	bottom left
	in_Tex:setPixel(EyesLocation[2].x+1, 	EyesLocation[2].y+1,	in_PixelData[8])	--	bottom right
	-- Push updated texture to GPU
	in_Tex:update()
end

-- Set the blink timer. If in_Duration is nil, will use configured blink timings.
function Eyes_Blink(in_Duration)
	-- This is the closest to an optional function argument Lua can get. Fuck's sake.
	BlinkShutTicker = in_Duration or BlinkDurationDay
	
	BlinkIntervalTicker = math.random(BlinkIntervalMin, BlinkIntervalMax) -- math.random() for integer output
end

-- LOGIC

-- On load (init function)
-- Bound to .init within Figura's Events API
function events.entity_init()
	-- model parts init
	
	-- props
	-- hide blahaj
	models.prop_blahaj:setVisible(false)
	models.prop_blahaj:setPos(3,18,-5)
	models.prop_blahaj:setRot(170,12,83)
	models.prop_blahaj.BodyGroup.FinR:setRot(60,0,0)
	models.prop_blahaj.BodyGroup.FinL:setRot(-60,0,0)


	-- Var init
	PlayerMaxHealth = player:getMaxHealth()
	PlayerHealth = player:getHealth()
	PlayerHunger = player:getFood()
	PlayerHealthProportion = PlayerHealth / PlayerMaxHealth
	
	Tex_Ruby01 = textures["ruby_01"]
	
	Sound_Howl = sounds["entity.wolf.howl"] -- Sound exists in stock but is unused
	Sound_Howl:attenuation(8) -- This is a multiplier. Effective range is value*16.
	Sound_Howl:subtitle("Wolf howls") -- No localisation string exists for this
	Sound_Howl:volume(0.75)
	
	Sound_Shakeoff = sounds["entity.wolf.shake"]
	Sound_Shakeoff:subtitle(client:getTranslatedString("subtitles.entity.wolf.shake"))
	
	
	-- Nameplate
	--nameplate.CHAT:setText("stick your username here maybe! :trans:")
	
	-- Eyes texture read
	-- Normal
	EyesPixelsOpen 		= Eyes_ReadPixels(vec(9,11),vec(13,11),Tex_Ruby01)
	EyesPixelsClosed 	= Eyes_ReadPixels(vec(62,16),vec(62,18),Tex_Ruby01)
	EyesPixelsLeft 		= Eyes_ReadPixels(vec(62,20),vec(62,22),Tex_Ruby01)
	EyesPixelsRight 	= Eyes_ReadPixels(vec(62,24),vec(62,26),Tex_Ruby01)
	EyesPixelsHalf 		= Eyes_ReadPixels(vec(62,28),vec(62,30),Tex_Ruby01)
	EyesPixelsHuge 		= Eyes_ReadPixels(vec(62,32),vec(62,34),Tex_Ruby01)
	-- Absorption
	EyesPixelsOpen_Uber 	= Eyes_ReadPixels(vec(62,36),vec(62,38),Tex_Ruby01)
	EyesPixelsLeft_Uber 	= Eyes_ReadPixels(vec(62,40),vec(62,42),Tex_Ruby01)
	EyesPixelsRight_Uber 	= Eyes_ReadPixels(vec(62,44),vec(62,46),Tex_Ruby01)
	EyesPixelsHalf_Uber 	= Eyes_ReadPixels(vec(62,48),vec(62,50),Tex_Ruby01)
	EyesPixelsHuge_Uber 	= Eyes_ReadPixels(vec(62,52),vec(62,54),Tex_Ruby01)
	
	
	-- ActionWheel setup
	local temp_AWPage1 = action_wheel:newPage()		-- New ActionWheel page
	action_wheel:setPage(temp_AWPage1)				-- Register new page to wheel
	-- Bark action
	temp_AWPage1:newAction()						-- New action within page
		-- Function chaining (action setup)
		:title("Bark")
		:onLeftClick(pings.Action_Bark)
		:setColor(NormaliseRGB(249,169,208))
		:setItem("minecraft:oak_log")
	-- Howl action
	temp_AWPage1:newAction()						-- New action within page
		-- Function chaining (action setup)
		:title("Howl")
		:onLeftClick(pings.Action_Howl)
		:setColor(NormaliseRGB(16,23,70))
		:setItem("minecraft:snowball")
	-- Pwease? action
	temp_AWPage1:newAction()						-- New action within page
		-- Function chaining (action setup)
		:title("Pwease?")
		:onToggle(pings.Action_Pwease)
		:setColor(NormaliseRGB(50,90,96))
		:setToggleColor(1,1,1)
		:setItem("minecraft:ender_eye")
		:setToggleItem("minecraft:ender_pearl")
		
end

-- PINGS

-- Bark
function pings.Action_Bark()
	sounds:playSound("entity.wolf.ambient", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false)
end

-- Howl
function pings.Action_Howl()
	Sound_Howl:stop()
	Sound_Howl:play()
	Eyes_Blink(35)
end

-- Pwease toggle
function pings.Action_Pwease(in_bToggleState)
	bPwease = in_bToggleState
	if bPwease then
		WhineTicker = 10
		
		nameplate.ENTITY:setScale(3,2,3)
		nameplate.ENTITY:setText(":pleading:")
	else
		nameplate.ENTITY:setScale(1)
		nameplate.ENTITY:setText(DefaultNameText)
	end
end

-- Every tick (update function)
-- Bound to .tick within Figura's Events API
function events.tick()
	
	-- Sounds
	if Sound_Howl:isPlaying() then Sound_Howl:pos(player:getPos())end
	if Sound_Shakeoff:isPlaying() then Sound_Shakeoff:pos(player:getPos())end
	
	-- Equipped armour materials
	do
		-- no switch/case my massive wolf arse. fuck you lua
		-- Boots
		ArmourItemID = player:getItem(3):getID()
		if 		ArmourItemID == "minecraft:air" 					then EquippedArmourMaterials[1] = eArmourMaterials.NoArmour		--log("Boots: None")
		elseif 	ArmourItemID == "minecraft:leather_boots" 			then EquippedArmourMaterials[1] = eArmourMaterials.Leather		--log("Boots: Leather")
		elseif 	ArmourItemID == "minecraft:chainmail_boots" 		then EquippedArmourMaterials[1] = eArmourMaterials.Chainmail    --log("Boots: Chain")
		elseif 	ArmourItemID == "minecraft:iron_boots" 				then EquippedArmourMaterials[1] = eArmourMaterials.Iron      	--log("Boots: Iron")	
		elseif 	ArmourItemID == "minecraft:golden_boots" 			then EquippedArmourMaterials[1] = eArmourMaterials.Gold         --log("Boots: Gold")
		elseif 	ArmourItemID == "minecraft:diamond_boots" 			then EquippedArmourMaterials[1] = eArmourMaterials.Diamond      --log("Boots: Diamond")
		elseif 	ArmourItemID == "minecraft:netherite_boots" 		then EquippedArmourMaterials[1] = eArmourMaterials.Netherite    --log("Boots: Netherite")
		end
		--log("boots: ", ArmourItemID) -- debug
		-- Legs
		ArmourItemID = player:getItem(4):getID()
		if 		ArmourItemID == "minecraft:air" 					then EquippedArmourMaterials[2] = eArmourMaterials.NoArmour	  	--log("Legs: None")
		elseif 	ArmourItemID == "minecraft:leather_leggings" 		then EquippedArmourMaterials[2] = eArmourMaterials.Leather	  	--log("Legs: Leather")
		elseif 	ArmourItemID == "minecraft:chainmail_leggings" 		then EquippedArmourMaterials[2] = eArmourMaterials.Chainmail	--log("Legs: Chain")
		elseif 	ArmourItemID == "minecraft:iron_leggings" 			then EquippedArmourMaterials[2] = eArmourMaterials.Iron	      	--log("Legs: Iron")	
		elseif 	ArmourItemID == "minecraft:golden_leggings" 		then EquippedArmourMaterials[2] = eArmourMaterials.Gold	      	--log("Legs: Gold")
		elseif 	ArmourItemID == "minecraft:diamond_leggings" 		then EquippedArmourMaterials[2] = eArmourMaterials.Diamond	  	--log("Legs: Diamond")
		elseif 	ArmourItemID == "minecraft:netherite_leggings" 		then EquippedArmourMaterials[2] = eArmourMaterials.Netherite	--log("Legs: Netherite")
		end
		--log("legs: ", ArmourItemID) -- debug
		-- Chest
		ArmourItemID = player:getItem(5):getID()
		if 		ArmourItemID == "minecraft:air" 					then EquippedArmourMaterials[3] = eArmourMaterials.NoArmour    	--log("Chest: None")
		elseif 	ArmourItemID == "minecraft:leather_chestplate" 		then EquippedArmourMaterials[3] = eArmourMaterials.Leather    	--log("Chest: Leather")
		elseif 	ArmourItemID == "minecraft:chainmail_chestplate" 	then EquippedArmourMaterials[3] = eArmourMaterials.Chainmail  	--log("Chest: Chain")
		elseif 	ArmourItemID == "minecraft:iron_chestplate" 		then EquippedArmourMaterials[3] = eArmourMaterials.Iron       	--log("Chest: Iron")	
		elseif 	ArmourItemID == "minecraft:golden_chestplate" 		then EquippedArmourMaterials[3] = eArmourMaterials.Gold       	--log("Chest: Gold")
		elseif 	ArmourItemID == "minecraft:diamond_chestplate" 		then EquippedArmourMaterials[3] = eArmourMaterials.Diamond    	--log("Chest: Diamond")
		elseif 	ArmourItemID == "minecraft:netherite_chestplate" 	then EquippedArmourMaterials[3] = eArmourMaterials.Netherite  	--log("Chest: Netherite")
		end                                                                                                                         
		--log("chest: ", ArmourItemID) -- debug
		-- Helmet
		ArmourItemID = player:getItem(6):getID()
		if 		ArmourItemID == "minecraft:air" 					then EquippedArmourMaterials[4] = eArmourMaterials.NoArmour    	--log("Helmet: None")
		elseif 	ArmourItemID == "minecraft:leather_helmet" 			then EquippedArmourMaterials[4] = eArmourMaterials.Leather    	--log("Helmet: Leather")
		elseif 	ArmourItemID == "minecraft:chainmail_helmet" 		then EquippedArmourMaterials[4] = eArmourMaterials.Chainmail  	--log("Helmet: Chain")
		elseif 	ArmourItemID == "minecraft:iron_helmet" 			then EquippedArmourMaterials[4] = eArmourMaterials.Iron       	--log("Helmet: Iron")	
		elseif 	ArmourItemID == "minecraft:golden_helmet" 			then EquippedArmourMaterials[4] = eArmourMaterials.Gold       	--log("Helmet: Gold")
		elseif 	ArmourItemID == "minecraft:diamond_helmet" 			then EquippedArmourMaterials[4] = eArmourMaterials.Diamond    	--log("Helmet: Diamond")
		elseif 	ArmourItemID == "minecraft:netherite_helmet" 		then EquippedArmourMaterials[4] = eArmourMaterials.Netherite  	--log("Helmet: Netherite")
		elseif 	ArmourItemID == "minecraft:turtle_helmet" 			then EquippedArmourMaterials[4] = eArmourMaterials.Turtle     	--log("Helmet: Turtle")
		end
		--log("helmet: ", ArmourItemID) -- debug
		--log("--------------------------------------") -- debug
	end
	
	
	-- HeldItem things
	-- Update vars
	ActiveItem = player:getActiveItem()
	if ActiveItem:getID() ~= "minecraft:air" then
		-- Food being eaten?
		if ActiveItem:isFood() then
			bIsEatingOrDrinking = true
		end
		
	else
		-- Reset state trackers
		bIsEatingOrDrinking = false
	end
	
	
	-- ###
	-- Eyes behaviour
	-- ###
	-- Eyes: Blink Overrides
	-- When sleeping, force eyes closed.
	if tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping then Eyes_Blink(20) end
	-- When dead, force eyes closed.
	if tPlayerEntityInfo.MoveState == ePlayerMoveState.Dead then Eyes_Blink(20) end
	-- Eyes: Blink logic
	-- Should eyes be shut?
	if BlinkShutTicker > 0 then
		-- Eyes should be shut.
		-- Decrement shut ticker
		BlinkShutTicker = BlinkShutTicker - 1
		-- Set shut flag
		bAreEyesShut = true
	else
		-- Eyes should be open
		-- Decrement interval ticker
		BlinkIntervalTicker = BlinkIntervalTicker - 1
		-- Set shut flag
		bAreEyesShut = false
		-- Check interval ticker is zero (or below)
		if BlinkIntervalTicker <= 0 then
			Eyes_Blink()
		end
	end
	
	
	-- Eyes: Non-blink Overrides
	if player:isBlocking() then EyesState = eEyesPose.HalfOpen goto Eyes_Checks_Done end
	if bPwease then EyesState = eEyesPose.Pwease goto Eyes_Checks_Done end
	
	-- Eyes: Eye pose (look direction) logic
	-- Is eye change cooldown ticker above zero?
	if EyeChangeTicker > 0 then
		-- Yes - decrement ticker, and skip rest of eye movement logic.
		EyeChangeTicker = EyeChangeTicker - 1
	else
		-- No - the eye state is allowed to change this tick.
		
		-- Dampener ticker check
		-- Is head moving quick enough?
		if math.abs(tPlayerEntityInfo.AngularVelocity.y) > LookSidewaysYawVelThreshold then
			-- Decrement ticker by head yaw avel (until 0)
			LookSidewaysAmountTicker = math.max(0, LookSidewaysAmountTicker - math.floor(math.abs(tPlayerEntityInfo.AngularVelocity.y)))
		else
			-- Not looking sideways enough - reset dampener
			LookSidewaysAmountTicker = LookSidewaysAmountThreshold
		end
		
		-- Is ticker completely saturated?
		if LookSidewaysAmountTicker == 0 then
			-- Make sure we're not already looking sideways
			if EyesState ~= eEyesPose.LookLeft and EyesState ~= eEyesPose.LookRight then
				-- Time to look sideways.
				
				-- Lock-in this eye pose for a little while, at least.
				EyeChangeTicker = EyeChangeCooldown
				-- Blink before large eye movements.
				Eyes_Blink()
				-- Set this here as to not have one frame of sideways before blinking
				bAreEyesShut = true
				-- Test left/right
				if tPlayerEntityInfo.AngularVelocity.y > 0 then
					-- Left
					EyesState = eEyesPose.LookLeft
				else
					-- Right
					EyesState = eEyesPose.LookRight
				end
				-- Left/right end
			end
		else
		
			-- At this point, looking sideways checks have failed. Player is looking ahead normally.
			-- If we're just returning to looking forward after a different eye pose, set the appropriate vars.
			if EyesState ~= eEyesPose.LookForward then
				EyesState = eEyesPose.LookForward
				EyeChangeTicker = EyeChangeCooldown
				-- Reduce blink ticker a little bit as well
				BlinkIntervalTicker = BlinkIntervalTicker - math.floor(BlinkIntervalMin*0.5)
			end
			
		end
		-- Look ticker saturated end
	end
	-- EyeChangeTicker end
	::Eyes_Checks_Done::
	-- Eyes: Run update func.
	-- Has built-in checks as to not perform useless work.
	Eyes_Update()
	-- Eyes: Debug
	--log("########","EYES","########")
	--log("cd ",EyeChangeTicker)
	--log("st",LookSidewaysAmountTicker,"\t(vel:",tPlayerEntityInfo.AngularVelocity.y,"/",LookSidewaysYawVelThreshold,")")
	--log("interval", BlinkIntervalTicker, "shut", BlinkShutTicker, "(", bAreEyesShut," / prev", bLastTickAreEyesShut, ")")
	
	-- End of Eyes behaviour
	-- ###
	
	-- Vel changes
	if tPlayerEntityInfo.Time ~= 0 then
		VelocityChange = tPlayerEntityInfo.Velocity - VelocityLastTick
		BodyYawAVelChange = tPlayerEntityInfo.BodyAngularVelocity - BodyYawAVelLastTick
	end
	
	
	-- Jiggle vars
	-- Jiggle from move vel delta
	if JiggleTickdown.y < math.abs(VelocityChange.y*JiggleDurationValue) then JiggleTickdown.y = math.min(JiggleTickerCap, math.ceil(math.abs(VelocityChange.y*JiggleDurationValue))) end
	if JiggleTickdown.x < math.abs(VelocityChange.x*JiggleDurationValue) then JiggleTickdown.x = math.min(JiggleTickerCap, math.ceil(math.abs(VelocityChange.x*JiggleDurationValue))) end
	if JiggleTickdown.z < math.abs(VelocityChange.z*JiggleDurationValue) then JiggleTickdown.z = math.min(JiggleTickerCap, math.ceil(math.abs(VelocityChange.z*JiggleDurationValue))) end
	-- Jiggle from body yaw avel delta
	if JiggleTickdown.x < math.abs(BodyYawAVelChange*JiggleBodyYawIntensity) then JiggleTickdown.x = math.min(JiggleTickerCap, math.ceil(math.abs(BodyYawAVelChange*JiggleBodyYawIntensity))) end
	if JiggleTickdown.z < math.abs(BodyYawAVelChange*JiggleBodyYawIntensity) then JiggleTickdown.z = math.min(JiggleTickerCap, math.ceil(math.abs(BodyYawAVelChange*JiggleBodyYawIntensity))) end
	-- Ticker decrement
	if JiggleTickdown.y > 0 then JiggleTickdown.y = JiggleTickdown.y - 1 end
	if JiggleTickdown.x > 0 then JiggleTickdown.x = JiggleTickdown.x - 1 end
	if JiggleTickdown.z > 0 then JiggleTickdown.z = JiggleTickdown.z - 1 end
	-- Cutoff
	if JiggleTickdown.y <= JiggleTickerCutoff then JiggleTickdown.y = 0 end
	if JiggleTickdown.x <= JiggleTickerCutoff then JiggleTickdown.x = 0 end
	if JiggleTickdown.z <= JiggleTickerCutoff then JiggleTickdown.z = 0 end
	
	
	-- Blahaj
	if tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping then
		-- Show Blahaj when sleeping
		models.prop_blahaj:setVisible(true)
	else
		-- Hide Blahaj when not sleeping
		models.prop_blahaj:setVisible(false)
	end
	
	
	-- Logic and Tickers
	
	-- Damage ticker decrement
	if TakenDamageTicker > 0 then TakenDamageTicker = TakenDamageTicker - 1 
	else TakenDamageTicker = 0
	end
	
	-- Shakeoff ticker decrement
	if ShakeoffTicker > 0 then ShakeoffTicker = ShakeoffTicker - 1 end
	if ShakeoffDelayTicker > 0 then ShakeoffDelayTicker = ShakeoffDelayTicker - 1 end
	
	-- Bark ticker
	if BarkTicker > 0 then
		BarkTicker = BarkTicker - 1
		if BarkTicker == 0 then -- On 0
			sounds:playSound("entity.wolf.ambient", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false)
			-- Push up other active tickers to prevent sound overlap
			--if BarkTicker > 0 then BarkTicker = BarkTicker + 20 end
			if GrowlTicker > 0 then GrowlTicker = GrowlTicker + 20 end
			if WhineTicker > 0 then WhineTicker = WhineTicker + 20 end
		end
	end
	-- Growl ticker
	if GrowlTicker > 0 then
		GrowlTicker = GrowlTicker - 1
		if GrowlTicker == 0 then -- On 0
			-- Dispatch sound
			sounds:playSound("entity.wolf.growl", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false)
			-- Push up other active tickers to prevent sound overlap
			if BarkTicker > 0 then BarkTicker = BarkTicker + 20 end
			--if GrowlTicker > 0 then GrowlTicker = GrowlTicker + 20 end
			if WhineTicker > 0 then WhineTicker = WhineTicker + 20 end
		end
	end
	-- Whine ticker
	if WhineTicker > 0 then
		WhineTicker = WhineTicker - 1
		if WhineTicker == 0 then -- On 0
			-- Dispatch sound
			sounds:playSound("entity.wolf.whine", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false)
			-- Push up other active tickers to prevent sound overlap
			if BarkTicker > 0 then BarkTicker = BarkTicker + 20 end
			if GrowlTicker > 0 then GrowlTicker = GrowlTicker + 20 end
			--if WhineTicker > 0 then WhineTicker = WhineTicker + 20 end
		end
	end
	
	-- Sprint ticker
	if tPlayerEntityInfo.MoveState == ePlayerMoveState.Sprinting then
		-- Increment while sprinting
		SprintTicker = SprintTicker + 1
	elseif SprintTicker > 0 then
		-- Decrement while not sprinting
		SprintTicker = SprintTicker - 1
		-- Check if sprinting for a while
		if SprintTicker > SprintPantTime then
			-- Random wolf panting sound dispatch
			if RandomWithinRange(0, SprintTicker) > (SprintPantTime + (SprintTicker*0.25)) then
				SprintTicker = 0
				sounds:playSound("entity.wolf.pant", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false)
			end
		end
	end
	
	-- Has player health gone down? 
	if PlayerHealth > player:getHealth() then
		-- Set damage ticker
		TakenDamageTicker = (PlayerHealth - player:getHealth()) * 20 -- Each point of damage extends ear tiltdown duration
		Eyes_Blink(10) -- Shut eyes for a moment
		
		-- Hurt sound
		-- This uses getHealth() because we need the new value Now, not next tick. Otherwise, this will play on death and overlap the death sound.
		if player:getHealth() ~= 0 then sounds:playSound("entity.wolf.hurt", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false) end
		-- Push up active tickers to prevent sound overlap
		if BarkTicker > 0 then BarkTicker = BarkTicker + 20 end
		if GrowlTicker > 0 then GrowlTicker = GrowlTicker + 20 end
		if WhineTicker > 0 then WhineTicker = WhineTicker + 20 end
	end
	
	-- Is player hurt, or is Pwease enabled?
	if WhineTicker <= 0 then
		if bPwease then
			WhineTicker = math.random(120,200) -- rand int
		elseif PlayerHealth < 12 then
			WhineTicker = math.random(LowHealthWhineMin,LowHealthWhineMax) -- rand int
		end
	elseif bPwease == false and PlayerHealth >= 12 then
		-- When conditions are failed, force whine ticker to -1 (to avoid late sound dispatch on 0)
		WhineTicker = -1
	end
	
	-- Has player eaten to full?
	-- Magic number: Max food bar is 20
	if player:getFood() == 20 and PlayerHunger < 20 then
		-- Set bark ticker. Bark later as to not overlap vanilla eating-finished burp sound.
		BarkTicker = 20
	end
	
	-- Is player blocking with shield?
	if player:isBlocking() then
		-- Growl occasionally
		if GrowlTicker <= 0 then GrowlTicker = math.random(80,180) end
	else
		-- When not blocking, force growl ticker to -1 (to avoid late sound dispatch on 0)
		GrowlTicker = -1
	end
	
	-- Wet check
	-- Is player water state NOT dry or in rain? or, is player state in rain and head+chest armour slots are empty?
	if (tPlayerEntityInfo.WaterState ~= ePlayerWaterState.Dry and tPlayerEntityInfo.WaterState ~= ePlayerWaterState.InRain) or (tPlayerEntityInfo.WaterState == ePlayerWaterState.InRain and (player:getItem(5):getID() == "minecraft:air" or player:getItem(6):getID() == "minecraft:air")) then
		-- Wet!
		bWetLastTick = true
		-- Just wading or fully soaked through?
		-- Includes being wet by rain
		if tPlayerEntityInfo.WaterState ~= ePlayerWaterState.Wading then bShakeoffFullyDrenched = true end
		-- If shaking off...
		if ShakeoffTicker > 0 or ShakeoffDelayTicker > -1 then
			-- Stop shaking off :3
			ShakeoffTicker = 0
			ShakeoffDelayTicker = -1
			Sound_Shakeoff:stop()
			Eyes_Blink(0)
		end
		else
		-- Dry!
		-- Was the player wet last tick?
		if bWetLastTick then
			-- Set delay ticker. Prepare to shake...!
			ShakeoffDelayTicker = ShakeoffDelayDuration
		end
		bWetLastTick = false
	end
	
	-- At end of shakeoff delay, shakeoff!
	if ShakeoffDelayTicker == 0 then
		ShakeoffDelayTicker = -1
		-- set anim control ticker
		ShakeoffTicker = ShakeoffDuration
		-- shake sfx
		Sound_Shakeoff:stop()
		Sound_Shakeoff:play()
		-- shut eyes
		Eyes_Blink(ShakeoffDuration)
		-- BCC: Disable script for a little bit
		BCC_DisableForTicks(ShakeoffDuration)
		-- Armour clink behaviour: Precalc and init
		ShakeoffArmourClinkTime = 20
		if EquippedArmourMaterials[1] ~= eArmourMaterials.NoArmour then ShakeoffArmourClinkTime = ShakeoffArmourClinkTime - 4 end -- Boots
		if EquippedArmourMaterials[2] ~= eArmourMaterials.NoArmour then ShakeoffArmourClinkTime = ShakeoffArmourClinkTime - 4 end -- Legs
		if EquippedArmourMaterials[3] ~= eArmourMaterials.NoArmour then ShakeoffArmourClinkTime = ShakeoffArmourClinkTime - 4 end -- Chest
		if EquippedArmourMaterials[4] ~= eArmourMaterials.NoArmour then ShakeoffArmourClinkTime = ShakeoffArmourClinkTime - 4 end -- Helmet
		-- If unchanged, no armour equipped - set to -1 instead to disable armour clink.
		if ShakeoffArmourClinkTime == 20 then ShakeoffArmourClinkTime = -1 end
		ShakeoffArmourClinkTicker = 0
		ShakeoffArmourClinkIndex = 1
		
		--log("shakeoff armour clink time =", ShakeoffArmourClinkTime)
	end
	-- While wet, drip
	if ShakeoffDelayTicker > 0 then
		
		-- Sopping fucking wet?
		if bShakeoffFullyDrenched == true then
			-- Sogged!
			particles:newParticle(
				"falling_dripstone_water", -- this particle has sounds attached!!! that's cool as fuck!!!
				player:getPos() + vec(
					RandomWithinRange(ShakeoffDripsVFXBounds.x*-1, ShakeoffDripsVFXBounds.x),
					RandomWithinRange(0,ShakeoffDripsVFXBounds.z)+ShakeoffDripsVFXBounds.w,
					RandomWithinRange(ShakeoffDripsVFXBounds.y*-1, ShakeoffDripsVFXBounds.y))
				)
		else
			-- Less sogged!
			if bShakeoffDripsSlowFlipFlop then
				particles:newParticle(
					"falling_water",
					player:getPos() + vec(
						RandomWithinRange(ShakeoffDripsVFXBounds.x*-1, ShakeoffDripsVFXBounds.x),
						RandomWithinRange(0,ShakeoffDripsVFXBounds.z*0.5)+ShakeoffDripsVFXBounds.w,
						RandomWithinRange(ShakeoffDripsVFXBounds.y*-1, ShakeoffDripsVFXBounds.y))
					)
				bShakeoffDripsSlowFlipFlop = false
			else
				bShakeoffDripsSlowFlipFlop = true
			end
		end
	end
	-- While shaking
	if ShakeoffTicker > 0 then
		-- Armour Clink logic
		-- Check if clinking enabled
		if ShakeoffArmourClinkTime ~= -1 then
			-- Ticker check
			if ShakeoffArmourClinkTicker > 0 then
				-- Decrement ticker
				ShakeoffArmourClinkTicker = ShakeoffArmourClinkTicker - 1
			else
				-- Ticker decremented
				-- Reset ticker
				ShakeoffArmourClinkTicker = ShakeoffArmourClinkTime
				-- Armour clink logic
				local temp_ArmourMat = EquippedArmourMaterials[ShakeoffArmourClinkIndex]
				local temp_SFXName = ""
				if 		temp_ArmourMat == eArmourMaterials.Leather		then temp_SFXName = "item.armor.equip_leather"
				elseif 	temp_ArmourMat == eArmourMaterials.Chainmail	then temp_SFXName = "item.armor.equip_chain"
				elseif 	temp_ArmourMat == eArmourMaterials.Iron			then temp_SFXName = "item.armor.equip_iron"
				elseif 	temp_ArmourMat == eArmourMaterials.Gold			then temp_SFXName = "item.armor.equip_gold"
				elseif 	temp_ArmourMat == eArmourMaterials.Diamond		then temp_SFXName = "item.armor.equip_diamond"
				elseif 	temp_ArmourMat == eArmourMaterials.Netherite	then temp_SFXName = "item.armor.equip_netherite"
				elseif 	temp_ArmourMat == eArmourMaterials.Turtle		then temp_SFXName = "item.armor.equip_turtle"
				end
				-- Iterate index
				ShakeoffArmourClinkIndex = ShakeoffArmourClinkIndex + 1
				-- Index rollover above 4
				if ShakeoffArmourClinkIndex > 4 then ShakeoffArmourClinkIndex = 1 end
				-- Keep iterating until reaching another armour piece
				while EquippedArmourMaterials[ShakeoffArmourClinkIndex] == eArmourMaterials.NoArmour do
					ShakeoffArmourClinkIndex = ShakeoffArmourClinkIndex + 1
				end
				-- All set up for the next sound now :3
				
				--log(temp_SFXName, "at", ShakeoffArmourClinkIndex)
				
				-- Dispatch sound
				sounds:playSound(temp_SFXName, player:getPos(), AudioVolume*0.66, RandomWithinRange(0.8, 1.2), false)
			end
		end
		
	
		-- Water droplet logic
		-- Spawn a bunch of droplets (decreasing with shakeoff progress)
		-- (Spawn less when not Sopping Fucking Wet)
		if bShakeoffFullyDrenched == true then
			-- SOGGED! Loads of water drops
			for i=0, math.max(1,math.floor(ShakeoffTicker*0.5)), 1 do
				particles:newParticle(
					"splash",
					player:getPos() + vec(
						RandomWithinRange(ShakeoffDripsVFXBounds.x*-1, ShakeoffDripsVFXBounds.x),
						RandomWithinRange(0,ShakeoffDripsVFXBounds.z)+ShakeoffDripsVFXBounds.w,
						RandomWithinRange(ShakeoffDripsVFXBounds.y*-1, ShakeoffDripsVFXBounds.y))
					)
			end
		else
			-- Less water
			for i=0, math.max(1,math.floor(ShakeoffTicker*0.25)), 1 do
				particles:newParticle(
					"splash",
					player:getPos() + vec(
						RandomWithinRange(ShakeoffDripsVFXBounds.x*-1, ShakeoffDripsVFXBounds.x),
						RandomWithinRange(0,ShakeoffDripsVFXBounds.z*0.5)+ShakeoffDripsVFXBounds.w,
						RandomWithinRange(ShakeoffDripsVFXBounds.y*-1, ShakeoffDripsVFXBounds.y))
					)
			end
		end
	end
	
	-- At end of shakeoff, set jiggle tickers hehe
	-- Also clear sopping state
	if ShakeoffTicker == 1 then
		JiggleTickdown = vec(JiggleTickerCap * 0.4, JiggleTickerCap * 0.5, JiggleTickerCap * 0.4)
		bShakeoffFullyDrenched = false
	end
	
	-- Sparkle particles during Pwease state
	if bPwease then
		-- Ticker
		if PweaseVFXTicker > 0 then
			PweaseVFXTicker = PweaseVFXTicker - 1
		else
			-- Ticker elapsed
			PweaseVFXTicker = math.random(1,15)
			-- spawn a little shine particle
			local NewParticle = particles:newParticle("end_rod", player:getPos() + vec(AwayFromZero(RandomWithinRange(-0.25, 0.25),0.25),RandomWithinRange(1.4, 2.1),AwayFromZero(RandomWithinRange(-0.25, 0.25),0.25)))
			NewParticle:scale(0.1)
		end
	end
	
	-- Ash particles during absorption uber
	if player:getAbsorptionAmount() > 0 then
		-- Ticker
		if AbsorptionVFXTicker > 0 then
			AbsorptionVFXTicker = AbsorptionVFXTicker - 1
		else
			-- Ticker elapsed
			AbsorptionVFXTicker = math.random(1,2)
			-- spawn ash particcle
			local NewParticle = particles:newParticle("ash", player:getPos() + vec(AwayFromZero(RandomWithinRange(-0.25, 0.25),0.75),RandomWithinRange(0.1, 0.4),AwayFromZero(RandomWithinRange(-0.25, 0.25),0.75)))
			NewParticle:setGravity(-0.1)
			NewParticle:setVelocity((NewParticle:getPos() - player:getPos()) * 0.025)
			
		end
	end
		
	
	
	-- Last-tick value reads
	-- Update stored movestate value
	MoveStateLastTick = tPlayerEntityInfo.MoveState
	-- Update stored health value
	PlayerHealth = player:getHealth()
	-- Update health proportion
	PlayerHealthProportion = PlayerHealth / PlayerMaxHealth
	-- Update stored absorption
	LastTickAbsorption = player:getAbsorptionAmount()
	-- Update stored hunger value
	PlayerHunger = player:getFood()
	-- Update stored vel value
	VelocityLastTick = tPlayerEntityInfo.Velocity
	-- Update stored body yaw avel value
	BodyYawAVelLastTick = tPlayerEntityInfo.BodyAngularVelocity
	-- Update stored eyestate values
	LastTickEyesState = EyesState
	bLastTickAreEyesShut = bAreEyesShut

	-- Death sound
	if player:getDeathTime() == 1 then sounds:playSound("entity.wolf.death", player:getPos(), AudioVolume, RandomWithinRange(AudioWolfPitchMin, AudioWolfPitchMax), false) end
	
end
-- end of tick()



-- LIBRARY: pont animationsystem
do
	
	-- Reset things here that don't have their own animation definition blocks
	-- Model Root, Thighs, Upperarms
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf, models.pinkwolf.Head, models.pinkwolf.Body,
			models.pinkwolf.LeftLeg.LeftThigh, models.pinkwolf.RightLeg.RightThigh, models.pinkwolf.LeftArm.LeftUpperarm, models.pinkwolf.RightArm.RightUpperarm
			},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	
	
	-- Ears
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Head.leftEar, models.pinkwolf.Head.rightEar},
		function() return true end,
		function(pos, rot) return pos, vec(0,0,0) end
	)
	-- Lower ears when taking damage
	DefineAnimation(
		{models.pinkwolf.Head.leftEar},
		function() return TakenDamageTicker ~= 0 end,
		function(pos, rot) return pos, vec(-35,-30,-16) end
	)
	DefineAnimation(
		{models.pinkwolf.Head.rightEar},
		function() return TakenDamageTicker ~= 0 end,
		function(pos, rot) return pos, vec(-35,30,16) end
	)
	-- Tilt ears while sneaking
	DefineAnimation(
		{models.pinkwolf.Head.leftEar},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, vec(-15,-15,-16) end
	)
	DefineAnimation(
		{models.pinkwolf.Head.rightEar},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, vec(-15,15,16) end
	)
	-- Ears back while shield blocking
	DefineAnimation(
		{models.pinkwolf.Head.leftEar},
		function() return player:isBlocking() end,
		function(pos, rot) return pos, vec(-70,-135,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Head.rightEar},
		function() return player:isBlocking() end,
		function(pos, rot) return pos, vec(-70,135,0) end
	)
	-- Ears down while pwease state is active
	DefineAnimation(
		{models.pinkwolf.Head.leftEar},
		function() return bPwease end,
		function(pos, rot) return pos, vec(-70,-20,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Head.rightEar},
		function() return bPwease end,
		function(pos, rot) return pos, vec(-70,20,0) end
	)
	-- Ears head pitch response
	DefineAnimation(
		{models.pinkwolf.Head.leftEar, models.pinkwolf.Head.rightEar},
		function() return true end,
		function(pos, rot) return pos, rot + vec(tPlayerEntityInfo.AngularVelocity.x*-0.6,0,0) end
	)
	
	-- Long hair (Shoulders)
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Body.HairShoulders},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Sneak pose
	DefineAnimation(
		{models.pinkwolf.Body.HairShoulders},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0,0.25,0), vec(8,0,0) end
	)
	
	-- Long hair (Back)
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Body.HairBack},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Vertical velocity response
	DefineAnimation(
		{models.pinkwolf.Body.HairBack},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly end,
		function(pos, rot) return vec(0,0,0), rot + vec(	math.min(0,math.max(-70, tPlayerEntityInfo.Velocity.y*100 ) )  ,0,0) end
	)
	
	-- Collar 
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Body.Collar},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Collar sneak pose
	DefineAnimation(
		{models.pinkwolf.Body.Collar},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0, 0, -0.5), vec(25,0,0) end
	)
	-- Collar tag lift when falling, and movement wiggle
	DefineAnimation(
		{models.pinkwolf.Body.Collar.tag},
		function() return true end,
		function(pos, rot) return pos, vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*-200,-20),75),0,(math.sin(tPlayerEntityInfo.Time*math.min(tPlayerEntityInfo.Velocity.xz:length()*3, 0.6))*tPlayerEntityInfo.Velocity.xz:length()*40)) end
	)
	-- Collar tag sneak offset
	-- Collar sneak pose
	DefineAnimation(
		{models.pinkwolf.Body.Collar.tag},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, rot + vec(45,0,0) end
	)
	
	-- Arms
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.RightArm},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,3) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftArm},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,-3) end
	)
	-- Sneak pose
	DefineAnimation(
		{models.pinkwolf.RightArm},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0,0,0), vec(0,-2,8) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftArm},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0,0,0), vec(0,2,-8) end
	)
	
	-- Boobs
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Body.Chest.Boobs},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Boobs sneak pose
	DefineAnimation(
		{models.pinkwolf.Body.Chest.Boobs},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0, 2, 0), vec(30,0,0) end
	)
	
	-- Tummy
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Tummy sneak pose
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0, 3.5, -2), vec(-12,0,0) end
	)
	
	-- Legs
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.RightLeg, models.pinkwolf.LeftLeg},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Legs sneak pose
	DefineAnimation(
		{models.pinkwolf.RightLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(0.8, 1, 0.7), vec(15,0,-7) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return vec(-0.8, 1, 0.7), vec(15,0,7) end
	)
	
	
	-- Tail wag (idle)
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		-- sin(sin((x*a)-b)+ sin((x*a)-b) * c) * d
		-- x is Time,
		-- a affects the time the curve takes (where higher multipliers are faster)
		-- b affects the offset of the curve
		-- c affects how flat the heights of the curve are (where 0 is just a sine curve, and 0.7ish is very flat. higher values get weird and 'bounce'.)
		-- d is the overall travel angle multiplier
		function(pos, rot) return pos, vec(0, (math.sin(math.sin((tPlayerEntityInfo.Time*0.15))+ (math.sin(tPlayerEntityInfo.Time*0.15)*0.2)))*10 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, vec(0, (math.sin(math.sin((tPlayerEntityInfo.Time*0.15)-1)+ (math.sin((tPlayerEntityInfo.Time*0.15)-1)*0.2)))*10 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.15)-1.5)*17,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.15)-2)*10,0) end
	)
	
	-- Tail wag (excited)
	-- Sneaking, Sprinting, Eating
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking or tPlayerEntityInfo.MoveState == ePlayerMoveState.Sprinting or bIsEatingOrDrinking end,
		function(pos, rot) return pos, vec(-23, (math.sin(math.sin((tPlayerEntityInfo.Time*0.75))+ (math.sin(tPlayerEntityInfo.Time*0.75)*0.2)))*20 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking or tPlayerEntityInfo.MoveState == ePlayerMoveState.Sprinting or bIsEatingOrDrinking end,
		function(pos, rot) return pos, vec(-8, (math.sin(math.sin((tPlayerEntityInfo.Time*0.75)-1)+ (math.sin((tPlayerEntityInfo.Time*0.75)-1)*0.2)))*30 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking or tPlayerEntityInfo.MoveState == ePlayerMoveState.Sprinting or bIsEatingOrDrinking end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.75)-1.5)*25,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking or tPlayerEntityInfo.MoveState == ePlayerMoveState.Sprinting or bIsEatingOrDrinking end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.75)-2)*14,0) end
	)
	
	-- Tail wag (slow)
	-- Wading
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.WaterState == ePlayerWaterState.Wading or player:isBlocking() end,
		-- sin(sin((x*a)-b)+ sin((x*a)-b) * c) * d
		-- x is Time,
		-- a affects the time the curve takes (where higher multipliers are faster)
		-- b affects the offset of the curve
		-- c affects how flat the heights of the curve are (where 0 is just a sine curve, and 0.7ish is very flat. higher values get weird and 'bounce'.)
		-- d is the overall travel angle multiplier
		function(pos, rot) return pos, vec(0, (math.sin(math.sin((tPlayerEntityInfo.Time*0.05))+ (math.sin(tPlayerEntityInfo.Time*0.05)*0.2)))*10 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.WaterState == ePlayerWaterState.Wading or player:isBlocking() end,
		function(pos, rot) return pos, vec(0, (math.sin(math.sin((tPlayerEntityInfo.Time*0.05)-1)+ (math.sin((tPlayerEntityInfo.Time*0.05)-1)*0.2)))*10 ,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.WaterState == ePlayerWaterState.Wading or player:isBlocking() end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.05)-1.5)*17,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.WaterState == ePlayerWaterState.Wading or player:isBlocking() end,
		function(pos, rot) return pos, vec(0,math.sin((tPlayerEntityInfo.Time*0.05)-2)*10,0) end
	)
	
	-- Tail prone offset
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Prone end,
		function(pos, rot) return pos, rot + vec(25,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Prone end,
		function(pos, rot) return pos, rot + vec(25,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Prone end,
		function(pos, rot) return pos, rot + vec(15,0,0) end
	)
	
	-- Tail blocking offset
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return player:isBlocking() end,
		function(pos, rot) return pos, rot + vec(15,0,0) end
	)
	
	-- Tail velocity response
	-- Pitch based on Y velocity
	-- Left/right based on body yaw delta
	-- Rules for tail base (excluding sneaking)
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sneaking and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Prone and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, rot + vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*60,-35),10), tPlayerEntityInfo.BodyAngularVelocity*-0.5,0) end
	)
	-- Rules for tail base (sneaking)
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, rot + vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*60,-5),10), tPlayerEntityInfo.BodyAngularVelocity*-0.5,0) end 
	)
	-- Rules for rest of tail
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Prone and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, rot + vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*80,-40),15), tPlayerEntityInfo.BodyAngularVelocity*-0.7,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Prone and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, rot + vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*40,-15),15), tPlayerEntityInfo.BodyAngularVelocity*-0.5,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Prone and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, rot + vec(math.min(math.max(tPlayerEntityInfo.Velocity.y*60,0),20), tPlayerEntityInfo.BodyAngularVelocity*-0.4,0) end
	)
	
	-- Tail health response
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sneaking and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Prone and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sitting end,
		function(pos, rot) return pos, rot + vec(20*(1-PlayerHealthProportion),0,0) end
	)
	
	-- Tail elytra flight animation
	-- flight val lives out here lol
	ElytraFlightAnimWiggle = 0
	-- Tail prone offset
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.ElytraFly end,
		function(pos, rot) ElytraFlightAnimWiggle = (math.sin(tPlayerEntityInfo.Time*math.max(tPlayerEntityInfo.Velocity:length(),0.8)*0.8)) return pos, vec(25, ElytraFlightAnimWiggle*10 , ElytraFlightAnimWiggle*20) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, vec(25,ElytraFlightAnimWiggle*-5,ElytraFlightAnimWiggle*-10) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, vec(20,ElytraFlightAnimWiggle*-2,ElytraFlightAnimWiggle*-4) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.ElytraFly end,
		function(pos, rot) return pos, vec(16,math.sin(tPlayerEntityInfo.Time*2)*10,0) end
	)
	
	
	-- Elytra
	-- Idle reset
	DefineAnimation(
		{vanilla_model.LEFT_ELYTRA, vanilla_model.RIGHT_ELYTRA},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Move out of the way of the tail
	DefineAnimation(
		{vanilla_model.LEFT_ELYTRA},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sneaking and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, vec(0,0,-20) end
	)
	DefineAnimation(
		{vanilla_model.RIGHT_ELYTRA},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.ElytraFly and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sneaking and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, vec(0,0,20) end
	)
	-- Move out of the way of the tail (sneaking)
	DefineAnimation(
		{vanilla_model.LEFT_ELYTRA},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, vec(-5,-25,10) end
	)
	DefineAnimation(
		{vanilla_model.RIGHT_ELYTRA},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return pos, vec(-5,25,-10) end
	)
	
	
	
	--####################################################################################################################################################################################
	--####################################################################################################################################################################################
	-- Shakeoff animation
	
	-- Lots of room for optimisation. cba if it isn't a problem though
	-- Global precalc vars
	--Shakeoff_SinTicker_MinusOnePointFive = math.sin((ShakeoffTicker)-1.5)
	
	-- Ears
	DefineAnimation(
		{models.pinkwolf.Head.leftEar, models.pinkwolf.Head.rightEar},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(-40,(math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*110,0) end
	)
	-- Head
	DefineAnimation(
		{models.pinkwolf.Head},
		function() return ShakeoffTicker > 0 end,
		-- sin(sin((x*a)-b)+ sin((x*a)-b) * c) * d
		-- x is shakeoff ticker,
		-- a affects the time the curve takes (where higher multipliers are faster)
		-- b affects the offset of the curve
		-- c affects how flat the heights of the curve are (where 0 is just a sine curve, and 0.7ish is very flat. higher values get weird and 'bounce'.)
		-- d is the overall travel angle multiplier
		function(pos, rot) return vec(0,-0.7,-3.7), vec(10, (math.sin(math.sin((ShakeoffTicker))+ (math.sin(ShakeoffTicker)*0.2)))*40 ,(math.sin(math.sin((ShakeoffTicker))+ (math.sin(ShakeoffTicker)*0.2)))*-20) end
	)
	
	-- Collar tag
	DefineAnimation(
		{models.pinkwolf.Body.Collar.tag},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(65,(math.sin(math.sin((ShakeoffTicker))+ ((math.sin(ShakeoffTicker))*0.2)))*30,0) end
	)
	-- Long hair (shoulders)
	DefineAnimation(
		{models.pinkwolf.Body.HairShoulders},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, rot + vec(20,0,0) end
	)
	-- Long hair (back)
	DefineAnimation(
		{models.pinkwolf.Body.HairBack},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, rot + vec(-20,0,(math.sin(math.sin((ShakeoffTicker))+ ((math.sin(ShakeoffTicker))*0.2)))*30) end
	)
	
	-- Body
	DefineAnimation(
		{models.pinkwolf.Body},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return vec(0,0,-3), vec(-15, (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*30 , (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*2) end
	)
	-- Boobs
	DefineAnimation(
		{models.pinkwolf.Body.Chest.Boobs},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return vec(0,0,0), rot+ vec(10, (math.sin(math.sin((ShakeoffTicker)-0.5)+ ((math.sin(ShakeoffTicker)-0.5)*0.2)))*30 ,0) end
	)
	-- Tummy
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return vec(0,0,1), rot+ vec(-8, (math.sin(math.sin((ShakeoffTicker)-0.75)+ ((math.sin(ShakeoffTicker)-0.75)*0.2)))*10 ,0) end
	)
	-- Elytra
	DefineAnimation(
		{vanilla_model.LEFT_ELYTRA},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return vec(0,0,-3), vec(-35, ((math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*-30)-45 , ((math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*2)-20) end
	)
	DefineAnimation(
		{vanilla_model.RIGHT_ELYTRA},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return vec(0,0,-3), vec(-35, ((math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*-30)+45 , ((math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*2)+20) end
	)
	
	-- Arms
	DefineAnimation(
		{models.pinkwolf.LeftArm},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return
		pos+ vec(0,0,-3.8 - (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*-3), 
		rot+ vec(-20, 25+(math.sin(math.sin((ShakeoffTicker)-2)+ ((math.sin(ShakeoffTicker)-2)*0.2)))*60 ,-15)
		end
	)
	DefineAnimation(
		{models.pinkwolf.RightArm},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return
		pos+ vec(0,0,-3.8 - (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*3), 
		rot+ vec(-20, -25-(math.sin(math.sin((ShakeoffTicker)-2)+ ((math.sin(ShakeoffTicker)-2)*0.2)))*-60 ,15)
		end
	)
	-- Arms (sneak adjust)
	DefineAnimation(
		{models.pinkwolf.LeftArm, models.pinkwolf.RightArm},
		function() return ShakeoffTicker > 0  and tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return
		pos+ vec(0,-1,1.5), 
		rot+ vec(rot.x*-0.7,0,rot.z*0.3)
		end
	)
	-- Upperarms
	DefineAnimation(
		{models.pinkwolf.LeftArm.LeftUpperarm},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, rot + vec(0, (math.sin(math.sin((ShakeoffTicker)+0.5)+ ((math.sin(ShakeoffTicker)+0.5)*0.2)))*25,0)
		end
	)
	DefineAnimation(
		{models.pinkwolf.RightArm.RightUpperarm},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, rot + vec(0, (math.sin(math.sin((ShakeoffTicker)+0.5)+ ((math.sin(ShakeoffTicker)+0.5)*0.2)))*25,0)
		end
	)
	
	-- Legs
	DefineAnimation(
		{models.pinkwolf.RightLeg, models.pinkwolf.LeftLeg},
		function() return ShakeoffTicker > 0 and tPlayerEntityInfo.Velocity.xz:length() < ShakeoffMoveVelThreshold end,
		function(pos, rot) return
		pos+ vec((math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-2)*0.2)))*2				,0,2), 
		rot+ vec(20, (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*25 		,0)
		end
	)
	-- Legs (Sneaking adjust)
	DefineAnimation(
		{models.pinkwolf.RightLeg, models.pinkwolf.LeftLeg},
		function() return ShakeoffTicker > 0 and tPlayerEntityInfo.Velocity.xz:length() < ShakeoffMoveVelThreshold and tPlayerEntityInfo.MoveState == ePlayerMoveState.Sneaking end,
		function(pos, rot) return
		pos+ vec(0,0,-2),
		rot+ vec(-10,0,0)
		end
	)
	-- Thighs
	DefineAnimation(
		{models.pinkwolf.RightLeg.RightThigh, models.pinkwolf.LeftLeg.LeftThigh},
		function() return ShakeoffTicker > 0 and tPlayerEntityInfo.Velocity.xz:length() < ShakeoffMoveVelThreshold end,
		function(pos, rot) return pos, rot + vec(0, (math.sin(math.sin((ShakeoffTicker)+0.3)+ ((math.sin(ShakeoffTicker)+0.3)*0.2)))*25,0)
		end
	)
	
	
	-- Tail
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(10, (math.sin(math.sin((ShakeoffTicker)-1.5)+ ((math.sin(ShakeoffTicker)-1.5)*0.2)))*-10, (math.sin(math.sin((ShakeoffTicker)+0.3)+ ((math.sin(ShakeoffTicker)+0.3)*0.2)))*20) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(10, (math.sin(math.sin((ShakeoffTicker)-1.0)+ ((math.sin(ShakeoffTicker)-1.0)*0.2)))*-15, (math.sin(math.sin((ShakeoffTicker)+0.8)+ ((math.sin(ShakeoffTicker)+0.8)*0.2)))*35) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(-10, (math.sin(math.sin((ShakeoffTicker)-0.5)+ ((math.sin(ShakeoffTicker)-0.5)*0.2)))*-5, (math.sin(math.sin((ShakeoffTicker)+1.3)+ ((math.sin(ShakeoffTicker)+1.3)*0.2)))*45) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return ShakeoffTicker > 0 end,
		function(pos, rot) return pos, vec(-10, (math.sin(math.sin((ShakeoffTicker))+ ((math.sin(ShakeoffTicker))*0.2)))*-5, (math.sin(math.sin((ShakeoffTicker)+1.8)+ ((math.sin(ShakeoffTicker)+1.8)*0.2)))*55) end
	)
	
	
	
	--####################################################################################################################################################################################
	--####################################################################################################################################################################################
	
	
	-- Jiggle movement response
	-- Boobs jiggle
	DefineAnimation(
		{models.pinkwolf.Body.Chest.Boobs},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, rot + vec(math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y)*JiggleTickdown.y*JiggleIntensityValue	))	,0,0) end
	)
	-- Tummy jiggle
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return pos, rot + vec(math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y*JiggleIntensityValue*1.1)*JiggleTickdown.y*JiggleIntensityValue*1.1	))	,0,0) end
	)
	-- Thighs jiggle
	DefineAnimation(
		{models.pinkwolf.LeftLeg.LeftThigh},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sitting end,
		function(pos, rot) return pos, rot + vec(0,math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.xz:length()*JiggleIntensityValue*0.9)*JiggleTickdown.xz:length()*JiggleIntensityValue*1.1	))	,0) end
	)
	DefineAnimation(
		{models.pinkwolf.RightLeg.RightThigh},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sitting end,
		function(pos, rot) return pos, rot + vec(0,math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.xz:length()*JiggleIntensityValue*0.9)*JiggleTickdown.xz:length()*JiggleIntensityValue*-1.1	))	,0) end
	)
	-- Upperarms jiggle
	DefineAnimation(
		{models.pinkwolf.LeftArm.LeftUpperarm},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sitting end,
		function(pos, rot) return pos, rot + vec(0,math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.xz:length()*JiggleIntensityValue*0.8)*JiggleTickdown.xz:length()*JiggleIntensityValue*0.9	))	,0) end
	)
	DefineAnimation(
		{models.pinkwolf.RightArm.RightUpperarm},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping and tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sitting end,
		function(pos, rot) return pos, rot + vec(0,math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.xz:length()*JiggleIntensityValue*0.8)*JiggleTickdown.xz:length()*JiggleIntensityValue*-0.9	))	,0) end
	)
	
	
	-- Sitting
	-- Move entire model up when sitting (cuz like. wow. huge thighs..)
	DefineAnimation(
		{models.pinkwolf},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,2,0), vec(0,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.RightLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0.8,0,1), vec(0,18,-10) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(-0.8,0,1), vec(0,-18,10) end
	)
	DefineAnimation(
		{models.pinkwolf.RightArm, vanilla_model.RIGHT_ARM},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,0,0), vec(0,0,5) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftArm, vanilla_model.LEFT_ARM},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,0,0), vec(0,0,-5) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,1,0.5), vec(5,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Chest.Boobs},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,0,0), vec(4,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.HairShoulders},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,0,0), vec(5,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sitting end,
		function(pos, rot) return vec(0,0,0), rot + vec(-20,0,0) end
	)
	
	
	-- Sleeping
	DefineAnimation(
		{models.pinkwolf},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(-2,0,-4), vec(0,-90,0) end
	)
	DefineAnimation(
		{models.pinkwolf.RightLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(-4,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftLeg},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(35,-20,10) end
	)
	DefineAnimation(
		{models.pinkwolf.RightArm},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,-1), vec(75,20,0) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftArm},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(55,-15,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tummy},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(25,0,-15) end
	)
	DefineAnimation(
		{models.pinkwolf.Head.rightEar},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(0,0,-15) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.HairShoulders},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(20,0,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(20,5,0) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(30,-15,(math.sin(tPlayerEntityInfo.Time*0.08))-5) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(70,-15,(math.sin((tPlayerEntityInfo.Time*0.08)-1)*2)-5) end
	)
	DefineAnimation(
		{models.pinkwolf.Body.Tail.Tail2.Tail3.Tail4},
		function() return tPlayerEntityInfo.MoveState == ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), vec(30,(math.sin((tPlayerEntityInfo.Time*0.08)-2)*5)+10,-5) end
	)
	-- Sleeping pose end
	
	
	-- Armour animations
	
	-- Reset when nothing else is happening
	DefineAnimation(
		{models.pinkwolf.RightArm.RightUpperarm.Armour_R_Upperarm, models.pinkwolf.LeftArm.LeftUpperarm.Armour_L_Upperarm,
		models.pinkwolf.RightLeg.Katt_RLegArmour.RightFauld, models.pinkwolf.LeftLeg.Katt_LLegArmour.LeftFauld},
		function() return true end,
		function(pos, rot) return vec(0,0,0), vec(0,0,0) end
	)
	-- Pauldrons
	-- Vertical velocity response
	DefineAnimation(
		{models.pinkwolf.RightArm.RightUpperarm.Armour_R_Upperarm},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), rot + vec(0,0,(math.max(-10,math.min(25, tPlayerEntityInfo.Velocity.y*-50))) + math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y*1.5)*(math.max(0,JiggleTickdown.y-10))*JiggleIntensityValue	))*0.5) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftArm.LeftUpperarm.Armour_L_Upperarm},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), rot + vec(0,0,(math.min(10,math.max(-25, tPlayerEntityInfo.Velocity.y*50))) - math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y*1.5)*(math.max(0,JiggleTickdown.y-10))*JiggleIntensityValue	))*0.5) end
	)
	-- Faulds
	-- Vertical velocity response
	DefineAnimation(
		{models.pinkwolf.RightLeg.Katt_RLegArmour.RightFauld},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), rot + vec(0,0,(math.max(-20,math.min(35, tPlayerEntityInfo.Velocity.y*-80))) + math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y*1.3)*(math.max(0,JiggleTickdown.y-4))*JiggleIntensityValue	))*0.65) end
	)
	DefineAnimation(
		{models.pinkwolf.LeftLeg.Katt_LLegArmour.LeftFauld},
		function() return tPlayerEntityInfo.MoveState ~= ePlayerMoveState.Sleeping end,
		function(pos, rot) return vec(0,0,0), rot + vec(0,0,(math.min(20,math.max(-35, tPlayerEntityInfo.Velocity.y*80))) - math.min(JiggleAngleClamp,math.max(	JiggleAngleClamp*-1, math.sin(JiggleTickdown.y*1.3)*(math.max(0,JiggleTickdown.y-4))*JiggleIntensityValue	))*0.65) end
	)
	
	
	end
-- animsys block end



-- LIBRARY: katt armor
-- https://github.com/KitCat962/Figura-KattArmor


-- Katt Armour API Config

local kattArmor = require("KattArmor")()
kattArmor.Armor.Helmet
-- the `addParts` function is not strict with the number of ModelParts provided. Add or remove parts as desired.
    :addParts(
		models.pinkwolf.Head.Katt_Helmet.Helmet,			-- Helmet
		models.pinkwolf.Head.Katt_Helmet.Helmet_snout,		-- Helmet snout part
		models.pinkwolf.Head.Katt_Helmet.HelmetHatLayer		-- Helmet (hat layer)
	  
    )
-- trims need a seperate cube to correctly function, so they get registered seperatly.
    :addTrimParts(
		models.pinkwolf.Head.Katt_Helmet.HelmetTrim,		-- Helmet trim
		models.pinkwolf.Head.Katt_Helmet.HelmetHatTrim		-- Helmet (hat layer) trim
    )
kattArmor.Armor.Chestplate
    :addParts(
       models.pinkwolf.Body.Chest.ChestArmour.Chestplate_Upper,					-- Chest armour
	   models.pinkwolf.Body.Chest.Boobs.BoobsArmour.Chestplate_Boobs,			-- Boobs armour
       models.pinkwolf.Body.Tail.Tail2.Tail2Armour.TailArmour2,					-- Tailplate2
       models.pinkwolf.Body.Tail.Tail2.Tail3.Tail3Armour.TailArmour3,			-- Tailplate3
       models.pinkwolf.RightArm.RightUpperarm.Armour_R_Upperarm.RightArmArmour,	-- Right pauldron
       models.pinkwolf.RightArm.RightForearm.Armour_R_Lowerarm.RightArmArmour,	-- Right gloves
       models.pinkwolf.LeftArm.LeftUpperarm.Armour_L_Upperarm.LeftArmArmour,	-- Left pauldron
       models.pinkwolf.LeftArm.LeftForearm.Armour_L_Lowerarm.LeftArmArmour		-- Left gloves
    )
    :addTrimParts(
       models.pinkwolf.Body.Chest.ChestArmour.Chestplate_UpperTrim,					-- Chest armour trim
	   models.pinkwolf.Body.Chest.Boobs.BoobsArmour.Chestplate_BoobsTrim,			-- Boobs armour trim
       models.pinkwolf.Body.Tail.Tail2.Tail2Armour.TailArmour2Trim,					-- Tailplate2 trim
       models.pinkwolf.Body.Tail.Tail2.Tail3.Tail3Armour.TailArmour3Trim,			-- Tailplate3 trim
       models.pinkwolf.RightArm.RightUpperarm.Armour_R_Upperarm.RightArmArmourTrim,	-- Right pauldron trim
       models.pinkwolf.RightArm.RightForearm.Armour_R_Lowerarm.RightArmArmourTrim,	-- Right gloves trim
       models.pinkwolf.LeftArm.LeftUpperarm.Armour_L_Upperarm.LeftArmArmourTrim,	-- Left pauldron trim
       models.pinkwolf.LeftArm.LeftForearm.Armour_L_Lowerarm.LeftArmArmourTrim		-- Left gloves trim
    )
kattArmor.Armor.Leggings
    :addParts(
       models.pinkwolf.Body.Tummy.BeltArmour.Belt,								-- Belt
       models.pinkwolf.RightLeg.Katt_RLegArmour.RightFauld.RightFauld,			-- Right fauld
       models.pinkwolf.LeftLeg.Katt_LLegArmour.LeftFauld.LeftFauld,				-- Left fauld
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmour,			-- Right greave
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmour2,			-- Right greave 2
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmour,				-- Left greave
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmour2				-- Left greave 2
    )
    :addTrimParts(
       models.pinkwolf.Body.Tummy.BeltArmour.BeltTrim,								-- Belt trim
       models.pinkwolf.RightLeg.Katt_RLegArmour.RightFauld.RightFauldTrim,			-- Right fauld trim
       models.pinkwolf.LeftLeg.Katt_LLegArmour.LeftFauld.LeftFauldTrim,				-- Left fauld trim
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmourTrim,			-- Right greave trim
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmour2Trim,			-- Right greave 2 trim
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmourTrim,				-- Left greave trim
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmour2Trim				-- Left greave 2 trim
    )
kattArmor.Armor.Boots
    :addParts(
       models.pinkwolf.RightLeg.Katt_RLegArmour.R_Boot.RBoot,			-- Right boot
       models.pinkwolf.LeftLeg.Katt_LLegArmour.L_Boot.LBoot				-- Left boot
    )
    :addTrimParts(
       models.pinkwolf.RightLeg.Katt_RLegArmour.R_Boot.RBootTrim,		-- Right boot trim
       models.pinkwolf.LeftLeg.Katt_LLegArmour.L_Boot.LBootTrim			-- Left boot trim
    )

-- This adds the brown parts of the leather armor that do not change color.
-- Because the texture path is long, store it in a variable.
local leather1 = "minecraft:textures/models/armor/leather_layer_1_overlay.png"
local leather2 = "minecraft:textures/models/armor/leather_layer_2_overlay.png"
kattArmor.Materials.leather
    :addParts("Helmet",
      -- Because this template aims to mimic the vanilla armor as much as possible, I'm setting the texture of the leather cubes in the bbmodel to the vanilla texture.
       models.pinkwolf.Head.Katt_Helmet.LeatherLayer									:setPrimaryTexture("RESOURCE", leather1),	-- Helmet
       models.pinkwolf.Head.Katt_Helmet.LeatherHatLayer									:setPrimaryTexture("RESOURCE", leather1)	-- Helmet (hat layer)
    )
    :addParts("Chestplate",
       models.pinkwolf.Body.Chest.ChestArmour.Chestplate_UpperLeather					:setPrimaryTexture("RESOURCE", leather1),	-- Chest armour
	   models.pinkwolf.Body.Chest.Boobs.BoobsArmour.Chestplate_BoobsLeather				:setPrimaryTexture("RESOURCE", leather1),	-- Boobs armour
       models.pinkwolf.Body.Tail.Tail2.Tail2Armour.TailArmour2Leather					:setPrimaryTexture("RESOURCE", leather1),	-- Tailplate2
       models.pinkwolf.Body.Tail.Tail2.Tail3.Tail3Armour.TailArmour3Leather				:setPrimaryTexture("RESOURCE", leather1),	-- Tailplate3
       models.pinkwolf.RightArm.RightUpperarm.Armour_R_Upperarm.RightArmArmourLeather	:setPrimaryTexture("RESOURCE", leather1),	-- Right pauldron
       models.pinkwolf.RightArm.RightForearm.Armour_R_Lowerarm.RightArmArmourLeather	:setPrimaryTexture("RESOURCE", leather1),	-- Right gloves
       models.pinkwolf.LeftArm.LeftUpperarm.Armour_L_Upperarm.LeftArmArmourLeather		:setPrimaryTexture("RESOURCE", leather1),	-- Left pauldron
       models.pinkwolf.LeftArm.LeftForearm.Armour_L_Lowerarm.LeftArmArmourLeather		:setPrimaryTexture("RESOURCE", leather1)	-- Left gloves
    )
	-- (Legs use armour image 2)
    :addParts("Leggings",
       models.pinkwolf.Body.Tummy.BeltArmour.BeltLeather								:setPrimaryTexture("RESOURCE", leather2),	-- Belt
       models.pinkwolf.RightLeg.Katt_RLegArmour.RightFauld.RightFauldLeather			:setPrimaryTexture("RESOURCE", leather2),	-- Right fauld
       models.pinkwolf.LeftLeg.Katt_LLegArmour.LeftFauld.LeftFauldLeather				:setPrimaryTexture("RESOURCE", leather2),	-- Left fauld
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmourLeather			:setPrimaryTexture("RESOURCE", leather2),	-- Right greave
       models.pinkwolf.RightLeg.RightThigh.R_LegArmour.RightLegArmour2Leather			:setPrimaryTexture("RESOURCE", leather2),	-- Right greave 2
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmourLeather				:setPrimaryTexture("RESOURCE", leather2),	-- Left greave
       models.pinkwolf.LeftLeg.LeftThigh.L_LegArmour.LeftLegArmour2Leather				:setPrimaryTexture("RESOURCE", leather2)	-- Left greave 2
    )
    :addParts("Boots",
       models.pinkwolf.RightLeg.Katt_RLegArmour.R_Boot.RBootLeather						:setPrimaryTexture("RESOURCE", leather1),	-- Right boot
       models.pinkwolf.LeftLeg.Katt_LLegArmour.L_Boot.LBootLeather						:setPrimaryTexture("RESOURCE", leather1)	-- Left boot
    )





